@@ -23,17 +23,16 @@
 namespace brpc {
 
 EventDispatcher::EventDispatcher()
-    : _epfd(-1)
+    : _event_dispatcher_fd(-1)
     , _stop(false)
     , _tid(0)
-    , _consumer_thread_attr(BTHREAD_ATTR_NORMAL)
-{
-    _epfd = kqueue();
-    if (_epfd < 0) {
+    , _thread_attr(BTHREAD_ATTR_NORMAL) {
+    _event_dispatcher_fd = kqueue();
+    if (_event_dispatcher_fd < 0) {
         PLOG(FATAL) << "Fail to create kqueue";
         return;
     }
-    CHECK_EQ(0, butil::make_close_on_exec(_epfd));
+    CHECK_EQ(0, butil::make_close_on_exec(_event_dispatcher_fd));
 
     _wakeup_fds[0] = -1;
     _wakeup_fds[1] = -1;
@@ -46,9 +45,9 @@ EventDispatcher::EventDispatcher()
 EventDispatcher::~EventDispatcher() {
     Stop();
     Join();
-    if (_epfd >= 0) {
-        close(_epfd);
-        _epfd = -1;
+    if (_event_dispatcher_fd >= 0) {
+        close(_event_dispatcher_fd);
+        _event_dispatcher_fd = -1;
     }
     if (_wakeup_fds[0] > 0) {
         close(_wakeup_fds[0]);
@@ -56,8 +55,8 @@ EventDispatcher::~EventDispatcher() {
     }
 }
 
-int EventDispatcher::Start(const bthread_attr_t* consumer_thread_attr) {
-    if (_epfd < 0) {
+int EventDispatcher::Start(const bthread_attr_t* thread_attr) {
+    if (_event_dispatcher_fd < 0) {
         LOG(FATAL) << "kqueue was not created";
         return -1;
     }
@@ -68,14 +67,13 @@ int EventDispatcher::Start(const bthread_attr_t* consumer_thread_attr) {
         return -1;
     }
 
-    // Set _consumer_thread_attr before creating kqueue thread to make sure
+    // Set _thread_attr before creating kqueue thread to make sure
     // everyting seems sane to the thread.
-    _consumer_thread_attr = (consumer_thread_attr  ?
-                             *consumer_thread_attr : BTHREAD_ATTR_NORMAL);
+    _thread_attr = (thread_attr ? *thread_attr : BTHREAD_ATTR_NORMAL);
 
-    //_consumer_thread_attr is used in StartInputEvent(), assign flag NEVER_QUIT to it will cause new bthread
+    //_thread_attr is used in StartInputEvent(), assign flag NEVER_QUIT to it will cause new bthread
     // that created by kevent() never to quit.
-    bthread_attr_t kqueue_thread_attr = _consumer_thread_attr | BTHREAD_NEVER_QUIT;
+    bthread_attr_t kqueue_thread_attr = _thread_attr | BTHREAD_NEVER_QUIT;
 
     // Polling thread uses the same attr for consumer threads (NORMAL right
     // now). Previously, we used small stack (32KB) which may be overflowed
@@ -92,17 +90,17 @@ int EventDispatcher::Start(const bthread_attr_t* consumer_thread_attr) {
 }
 
 bool EventDispatcher::Running() const {
-    return !_stop  && _epfd >= 0 && _tid != 0;
+    return !_stop  && _event_dispatcher_fd >= 0 && _tid != 0;
 }
 
 void EventDispatcher::Stop() {
     _stop = true;
 
-    if (_epfd >= 0) {
+    if (_event_dispatcher_fd >= 0) {
         struct kevent kqueue_event;
         EV_SET(&kqueue_event, _wakeup_fds[1], EVFILT_WRITE, EV_ADD | EV_ENABLE,
                     0, 0, NULL);
-        kevent(_epfd, &kqueue_event, 1, NULL, 0, NULL);
+        kevent(_event_dispatcher_fd, &kqueue_event, 1, NULL, 0, NULL);
     }
 }
 
@@ -113,8 +111,9 @@ void EventDispatcher::Join() {
     }
 }
 
-int EventDispatcher::RegisterEvent(SocketId socket_id, int fd, bool pollin) {
-    if (_epfd < 0) {
+int EventDispatcher::RegisterEvent(IOEventDataId event_data_id,
+                                   int fd, bool pollin) {
+    if (_event_dispatcher_fd < 0) {
         errno = EINVAL;
         return -1;
     }
@@ -122,44 +121,44 @@ int EventDispatcher::RegisterEvent(SocketId socket_id, int fd, bool pollin) {
     struct kevent evt;
     //TODO(zhujiashun): add EV_EOF
     EV_SET(&evt, fd, EVFILT_WRITE, EV_ADD | EV_ENABLE | EV_CLEAR,
-                0, 0, (void*)socket_id);
-    if (kevent(_epfd, &evt, 1, NULL, 0, NULL) < 0) {
+           0, 0, (void*)event_data_id);
+    if (kevent(_event_dispatcher_fd, &evt, 1, NULL, 0, NULL) < 0) {
         return -1;
     }
     if (pollin) {
         EV_SET(&evt, fd, EVFILT_READ, EV_ADD | EV_ENABLE | EV_CLEAR,
-                    0, 0, (void*)socket_id);
-        if (kevent(_epfd, &evt, 1, NULL, 0, NULL) < 0) {
+               0, 0, (void*)event_data_id);
+        if (kevent(_event_dispatcher_fd, &evt, 1, NULL, 0, NULL) < 0) {
             return -1;
         }
     }
     return 0;
 }
 
-int EventDispatcher::UnregisterEvent(SocketId socket_id, 
-                                    int fd, bool pollin) {
+int EventDispatcher::UnregisterEvent(IOEventDataId event_data_id,
+                                     int fd, bool pollin) {
     struct kevent evt;
     EV_SET(&evt, fd, EVFILT_WRITE, EV_DELETE, 0, 0, NULL);
-    if (kevent(_epfd, &evt, 1, NULL, 0, NULL) < 0) {
+    if (kevent(_event_dispatcher_fd, &evt, 1, NULL, 0, NULL) < 0) {
         return -1;
     }
     if (pollin) {
         EV_SET(&evt, fd, EVFILT_READ, EV_ADD | EV_ENABLE | EV_CLEAR,
-                    0, 0, (void*)socket_id);
-        return kevent(_epfd, &evt, 1, NULL, 0, NULL);
+               0, 0, (void*)event_data_id);
+        return kevent(_event_dispatcher_fd, &evt, 1, NULL, 0, NULL);
     }
     return 0;
 }
 
-int EventDispatcher::AddConsumer(SocketId socket_id, int fd) {
-    if (_epfd < 0) {
+int EventDispatcher::AddConsumer(IOEventDataId event_data_id, int fd) {
+    if (_event_dispatcher_fd < 0) {
         errno = EINVAL;
         return -1;
     }
     struct kevent evt;
     EV_SET(&evt, fd, EVFILT_READ, EV_ADD | EV_ENABLE | EV_CLEAR,
-                0, 0, (void*)socket_id);
-    return kevent(_epfd, &evt, 1, NULL, 0, NULL);
+           0, 0, (void*)event_data_id);
+    return kevent(_event_dispatcher_fd, &evt, 1, NULL, 0, NULL);
 }
 
 int EventDispatcher::RemoveConsumer(int fd) {
@@ -175,9 +174,9 @@ int EventDispatcher::RemoveConsumer(int fd) {
     // program abnormal.
     struct kevent evt;
     EV_SET(&evt, fd, EVFILT_READ, EV_DELETE, 0, 0, NULL);
-    kevent(_epfd, &evt, 1, NULL, 0, NULL);
+    kevent(_event_dispatcher_fd, &evt, 1, NULL, 0, NULL);
     EV_SET(&evt, fd, EVFILT_WRITE, EV_DELETE, 0, 0, NULL);
-    kevent(_epfd, &evt, 1, NULL, 0, NULL);
+    kevent(_event_dispatcher_fd, &evt, 1, NULL, 0, NULL);
     return 0;
 }
 
@@ -189,7 +188,7 @@ void* EventDispatcher::RunThis(void* arg) {
 void EventDispatcher::Run() {
     while (!_stop) {
         struct kevent e[32];
-        int n = kevent(_epfd, NULL, 0, e, ARRAY_SIZE(e), NULL);
+        int n = kevent(_event_dispatcher_fd, NULL, 0, e, ARRAY_SIZE(e), NULL);
         if (_stop) {
             // EV_SET/kevent should have some sort of memory fencing
             // guaranteeing that we(after kevent) see _stop set before
@@ -201,20 +200,21 @@ void EventDispatcher::Run() {
                 // We've checked _stop, no wake-up will be missed.
                 continue;
             }
-            PLOG(FATAL) << "Fail to kqueue epfd=" << _epfd;
+            PLOG(FATAL) << "Fail to kqueue epfd=" << _event_dispatcher_fd;
             break;
         }
         for (int i = 0; i < n; ++i) {
             if ((e[i].flags & EV_ERROR) || e[i].filter == EVFILT_READ) {
                 // We don't care about the return value.
-                Socket::StartInputEvent((SocketId)e[i].udata, e[i].filter,
-                                        _consumer_thread_attr);
+                CallInputEventCallback((IOEventDataId)e[i].udata,
+                                       e[i].filter, _thread_attr);
             }
         }
         for (int i = 0; i < n; ++i) {
             if ((e[i].flags & EV_ERROR) || e[i].filter == EVFILT_WRITE) {
                 // We don't care about the return value.
-                Socket::HandleEpollOut((SocketId)e[i].udata);
+                CallOutputEventCallback((IOEventDataId)e[i].udata,
+                                        e[i].filter, _thread_attr);
             }
         }
     }
