@@ -19,58 +19,41 @@
 #ifndef BRPC_NSHEAD_MESSAGE_H
 #define BRPC_NSHEAD_MESSAGE_H
 
-#include <google/protobuf/message.h>
-#include "brpc/nshead.h"                     // nshead_t
-#include "butil/iobuf.h"                     // IOBuf
-#include "brpc/proto_base.pb.h"
+#include "brpc/nonreflectable_message.h"
+#include "brpc/nshead.h" // nshead_t
 #include "brpc/pb_compat.h"
+#include "butil/iobuf.h" // IOBuf
 
 namespace brpc {
 
 // Representing a nshead request or response.
-class NsheadMessage : public ::google::protobuf::Message {
+class NsheadMessage : public NonreflectableMessage<NsheadMessage> {
 public:
     nshead_t head;
     butil::IOBuf body;
     
 public:
     NsheadMessage();
-    virtual ~NsheadMessage();
-  
+    ~NsheadMessage() override;
+
     NsheadMessage(const NsheadMessage& from);
-  
+
     inline NsheadMessage& operator=(const NsheadMessage& from) {
         CopyFrom(from);
         return *this;
     }
-  
-    static const ::google::protobuf::Descriptor* descriptor();
-  
-    void Swap(NsheadMessage* other);
-  
-    // implements Message ----------------------------------------------
-  
-    NsheadMessage* New() const PB_319_OVERRIDE;
-#if GOOGLE_PROTOBUF_VERSION >= 3006000
-    NsheadMessage* New(::google::protobuf::Arena* arena) const override;
-#endif
-    void CopyFrom(const ::google::protobuf::Message& from) PB_321_OVERRIDE;
-    void MergeFrom(const ::google::protobuf::Message& from) override;
-    void CopyFrom(const NsheadMessage& from);
-    void MergeFrom(const NsheadMessage& from);
-    void Clear() override;
-    bool IsInitialized() const override;
-  
-    int ByteSize() const;
-    bool MergePartialFromCodedStream(
-        ::google::protobuf::io::CodedInputStream* input) PB_310_OVERRIDE;
-    void SerializeWithCachedSizes(
-        ::google::protobuf::io::CodedOutputStream* output) const PB_310_OVERRIDE;
-    ::google::protobuf::uint8* SerializeWithCachedSizesToArray(::google::protobuf::uint8* output) const PB_310_OVERRIDE;
-    int GetCachedSize() const PB_422_OVERRIDE { return ByteSize(); }
 
-protected:
-    ::google::protobuf::Metadata GetMetadata() const override;
+    void Swap(NsheadMessage* other);
+
+    // implements Message ----------------------------------------------
+    void MergeFrom(const ::google::protobuf::Message& from) PB_526_OVERRIDE;
+    void MergeFrom(const NsheadMessage& from) override;
+    void Clear() override;
+
+    size_t ByteSizeLong() const override;
+    int GetCachedSize() const PB_425_OVERRIDE { return ByteSize(); }
+
+    ::google::protobuf::Metadata GetMetadata() const PB_527_OVERRIDE;
 
 private:
     void SharedCtor();
