@@ -35,7 +35,6 @@
 #include "bthread/task_control.h"
 #include "bthread/task_group.h"
 #include "bthread/timer_thread.h"
-#include "bthread/errno.h"
 
 namespace bthread {
 
@@ -45,8 +44,7 @@ static const bthread_attr_t BTHREAD_ATTR_TASKGROUP = {
 static bool pass_bool(const char*, bool) { return true; }
 
 DEFINE_bool(show_bthread_creation_in_vars, false, "When this flags is on, The time "
-            "from bthread creation to first run will be recorded and shown "
-            "in /vars");
+            "from bthread creation to first run will be recorded and shown in /vars");
 const bool ALLOW_UNUSED dummy_show_bthread_creation_in_vars =
     ::GFLAGS_NS::RegisterFlagValidator(&FLAGS_show_bthread_creation_in_vars,
                                     pass_bool);
@@ -57,6 +55,11 @@ const bool ALLOW_UNUSED dummy_show_per_worker_usage_in_vars =
     ::GFLAGS_NS::RegisterFlagValidator(&FLAGS_show_per_worker_usage_in_vars,
                                     pass_bool);
 
+DEFINE_bool(bthread_enable_cpu_clock_stat, false,
+            "Enable CPU clock statistics for bthread");
+const bool ALLOW_UNUSED dummy_bthread_enable_cpu_clock_stat =  ::GFLAGS_NS::RegisterFlagValidator(&FLAGS_bthread_enable_cpu_clock_stat,
+                                    pass_bool);
+
 BAIDU_VOLATILE_THREAD_LOCAL(TaskGroup*, tls_task_group, NULL);
 // Sync with TaskMeta::local_storage when a bthread is created or destroyed.
 // During running, the two fields may be inconsistent, use tls_bls as the
@@ -70,7 +73,7 @@ extern void return_keytable(bthread_keytable_pool_t*, KeyTable*);
 // overhead of creation keytable, may be removed later.
 BAIDU_VOLATILE_THREAD_LOCAL(void*, tls_unique_user_ptr, NULL);
 
-const TaskStatistics EMPTY_STAT = { 0, 0 };
+const TaskStatistics EMPTY_STAT = { 0, 0, 0 };
 
 const size_t OFFSET_TABLE[] = {
 #include "bthread/offset_inl.list"
@@ -183,8 +186,7 @@ void TaskGroup::run_main_task() {
 }
 
 TaskGroup::TaskGroup(TaskControl* c)
-    :
-    _cur_meta(NULL)
+    : _cur_meta(NULL)
     , _control(c)
     , _num_nosignal(0)
     , _nsignaled(0)
@@ -202,7 +204,7 @@ TaskGroup::TaskGroup(TaskControl* c)
     , _sched_recursive_guard(0)
 #endif
     , _tag(BTHREAD_TAG_DEFAULT)
-{
+    , _tid(-1) {
     _steal_seed = butil::fast_rand();
     _steal_offset = OFFSET_TABLE[_steal_seed % ARRAY_SIZE(OFFSET_TABLE)];
     CHECK(c);
@@ -255,6 +257,7 @@ int TaskGroup::init(size_t runqueue_capacity) {
     _main_tid = m->tid;
     _main_stack = stk;
     _last_run_ns = butil::cpuwide_time_ns();
+    _last_cpu_clock_ns = 0;
     return 0;
 }
 
@@ -262,6 +265,9 @@ void TaskGroup::task_runner(intptr_t skip_remained) {
     // NOTE: tls_task_group is volatile since tasks are moved around
     //       different groups.
     TaskGroup* g = tls_task_group;
+#ifdef BRPC_BTHREAD_TRACER
+    TaskTracer::set_running_status(g->tid(), g->_cur_meta);
+#endif // BRPC_BTHREAD_TRACER
 
     if (!skip_remained) {
         while (g->_last_context_remained) {
@@ -337,14 +343,28 @@ void TaskGroup::task_runner(intptr_t skip_remained) {
         // is 0, change it to 1 to make bthread_t never be 0. Any access
         // or join to the bthread after changing version will be rejected.
         // The spinlock is for visibility of TaskGroup::get_attr.
+#ifdef BRPC_BTHREAD_TRACER
+        bool tracing = false;
+#endif // BRPC_BTHREAD_TRACER
         {
             BAIDU_SCOPED_LOCK(m->version_lock);
+#ifdef BRPC_BTHREAD_TRACER
+            tracing = TaskTracer::set_end_status_unsafe(m);
+#endif // BRPC_BTHREAD_TRACER
             if (0 == ++*m->version_butex) {
                 ++*m->version_butex;
             }
         }
         butex_wake_except(m->version_butex, 0);
 
+#ifdef BRPC_BTHREAD_TRACER
+        if (tracing) {
+            // Wait for tracing completion.
+            g->_control->_task_tracer.WaitForTracing(m);
+        }
+        g->_control->_task_tracer.set_status(TASK_STATUS_UNKNOWN, m);
+#endif // BRPC_BTHREAD_TRACER
+
         g->_control->_nbthreads << -1;
         g->_control->tag_nbthreads(g->tag()) << -1;
         g->set_remained(TaskGroup::_release_last_context, m);
@@ -379,7 +399,7 @@ int TaskGroup::start_foreground(TaskGroup** pg,
     const bthread_attr_t using_attr = (attr ? *attr : BTHREAD_ATTR_NORMAL);
     butil::ResourceId<TaskMeta> slot;
     TaskMeta* m = butil::get_resource(&slot);
-    if (__builtin_expect(!m, 0)) {
+    if (BAIDU_UNLIKELY(NULL == m)) {
         return ENOMEM;
     }
     CHECK(m->current_waiter.load(butil::memory_order_relaxed) == NULL);
@@ -406,9 +426,12 @@ int TaskGroup::start_foreground(TaskGroup** pg,
     TaskGroup* g = *pg;
     g->_control->_nbthreads << 1;
     g->_control->tag_nbthreads(g->tag()) << 1;
+#ifdef BRPC_BTHREAD_TRACER
+    g->_control->_task_tracer.set_status(TASK_STATUS_CREATED, m);
+#endif // BRPC_BTHREAD_TRACER
     if (g->is_current_pthread_task()) {
         // never create foreground task in pthread.
-        g->ready_to_run(m->tid, (using_attr.flags & BTHREAD_NOSIGNAL));
+        g->ready_to_run(m, using_attr.flags & BTHREAD_NOSIGNAL);
     } else {
         // NOSIGNAL affects current task, not the new task.
         RemainedFn fn = NULL;
@@ -417,10 +440,7 @@ int TaskGroup::start_foreground(TaskGroup** pg,
         } else {
             fn = ready_to_run_in_worker;
         }
-        ReadyToRunArgs args = {
-            g->current_tid(),
-            (bool)(using_attr.flags & BTHREAD_NOSIGNAL)
-        };
+        ReadyToRunArgs args = { g->_cur_meta, (bool)(using_attr.flags & BTHREAD_NOSIGNAL) };
         g->set_remained(fn, &args);
         TaskGroup::sched_to(pg, m->tid);
     }
@@ -439,7 +459,7 @@ int TaskGroup::start_background(bthread_t* __restrict th,
     const bthread_attr_t using_attr = (attr ? *attr : BTHREAD_ATTR_NORMAL);
     butil::ResourceId<TaskMeta> slot;
     TaskMeta* m = butil::get_resource(&slot);
-    if (__builtin_expect(!m, 0)) {
+    if (BAIDU_UNLIKELY(NULL == m)) {
         return ENOMEM;
     }
     CHECK(m->current_waiter.load(butil::memory_order_relaxed) == NULL);
@@ -464,10 +484,13 @@ int TaskGroup::start_background(bthread_t* __restrict th,
     }
     _control->_nbthreads << 1;
     _control->tag_nbthreads(tag()) << 1;
+#ifdef BRPC_BTHREAD_TRACER
+    _control->_task_tracer.set_status(TASK_STATUS_CREATED, m);
+#endif // BRPC_BTHREAD_TRACER
     if (REMOTE) {
-        ready_to_run_remote(m->tid, (using_attr.flags & BTHREAD_NOSIGNAL));
+        ready_to_run_remote(m, (using_attr.flags & BTHREAD_NOSIGNAL));
     } else {
-        ready_to_run(m->tid, (using_attr.flags & BTHREAD_NOSIGNAL));
+        ready_to_run(m, (using_attr.flags & BTHREAD_NOSIGNAL));
     }
     return 0;
 }
@@ -489,7 +512,7 @@ int TaskGroup::join(bthread_t tid, void** return_value) {
         return EINVAL;
     }
     TaskMeta* m = address_meta(tid);
-    if (__builtin_expect(!m, 0)) {
+    if (BAIDU_UNLIKELY(NULL == m)) {
         // The bthread is not created yet, this join is definitely wrong.
         return EINVAL;
     }
@@ -602,6 +625,17 @@ void TaskGroup::sched_to(TaskGroup** pg, TaskMeta* next_meta) {
     const int64_t elp_ns = now - g->_last_run_ns;
     g->_last_run_ns = now;
     cur_meta->stat.cputime_ns += elp_ns;
+
+    if (FLAGS_bthread_enable_cpu_clock_stat) {
+        const int64_t cpu_thread_time = butil::cputhread_time_ns();
+        if (g->_last_cpu_clock_ns != 0) {
+            cur_meta->stat.cpu_usage_ns += cpu_thread_time - g->_last_cpu_clock_ns;
+        }
+        g->_last_cpu_clock_ns = cpu_thread_time;
+    } else {
+        g->_last_cpu_clock_ns = 0;
+    }
+    
     if (cur_meta->tid != g->main_tid()) {
         g->_cumulated_cputime_ns += elp_ns;
     }
@@ -625,9 +659,16 @@ void TaskGroup::sched_to(TaskGroup** pg, TaskMeta* next_meta) {
         if (cur_meta->stack != NULL) {
             if (next_meta->stack != cur_meta->stack) {
                 CheckBthreadScheSafety();
+#ifdef BRPC_BTHREAD_TRACER
+                g->_control->_task_tracer.set_status(TASK_STATUS_JUMPING, cur_meta);
+                g->_control->_task_tracer.set_status(TASK_STATUS_JUMPING, next_meta);
+#endif // BRPC_BTHREAD_TRACER
                 jump_stack(cur_meta->stack, next_meta->stack);
                 // probably went to another group, need to assign g again.
                 g = BAIDU_GET_VOLATILE_THREAD_LOCAL(tls_task_group);
+#ifdef BRPC_BTHREAD_TRACER
+                TaskTracer::set_running_status(g->tid(), g->_cur_meta);
+#endif // BRPC_BTHREAD_TRACER
             }
 #ifndef NDEBUG
             else {
@@ -636,8 +677,13 @@ void TaskGroup::sched_to(TaskGroup** pg, TaskMeta* next_meta) {
                 CHECK(cur_meta->stack == g->_main_stack);
             }
 #endif
+        } /* else because of ending_sched(including pthread_task->pthread_task). */
+#ifdef BRPC_BTHREAD_TRACER
+        else {
+            // _cur_meta: TASK_STATUS_FIRST_READY -> TASK_STATUS_RUNNING.
+            TaskTracer::set_running_status(g->tid(), g->_cur_meta);
         }
-        // else because of ending_sched(including pthread_task->pthread_task)
+#endif // BRPC_BTHREAD_TRACER
     } else {
         LOG(FATAL) << "bthread=" << g->current_tid() << " sched_to itself!";
     }
@@ -669,8 +715,12 @@ void TaskGroup::destroy_self() {
     }
 }
 
-void TaskGroup::ready_to_run(bthread_t tid, bool nosignal) {
-    push_rq(tid);
+
+void TaskGroup::ready_to_run(TaskMeta* meta, bool nosignal) {
+#ifdef BRPC_BTHREAD_TRACER
+    _control->_task_tracer.set_status(TASK_STATUS_READY, meta);
+#endif // BRPC_BTHREAD_TRACER
+    push_rq(meta->tid);
     if (nosignal) {
         ++_num_nosignal;
     } else {
@@ -690,9 +740,12 @@ void TaskGroup::flush_nosignal_tasks() {
     }
 }
 
-void TaskGroup::ready_to_run_remote(bthread_t tid, bool nosignal) {
+void TaskGroup::ready_to_run_remote(TaskMeta* meta, bool nosignal) {
+#ifdef BRPC_BTHREAD_TRACER
+    _control->_task_tracer.set_status(TASK_STATUS_READY, meta);
+#endif // BRPC_BTHREAD_TRACER
     _remote_rq._mutex.lock();
-    while (!_remote_rq.push_locked(tid)) {
+    while (!_remote_rq.push_locked(meta->tid)) {
         flush_nosignal_tasks_remote_locked(_remote_rq._mutex);
         LOG_EVERY_SECOND(ERROR) << "_remote_rq is full, capacity="
                                 << _remote_rq.capacity();
@@ -723,11 +776,11 @@ void TaskGroup::flush_nosignal_tasks_remote_locked(butil::Mutex& locked_mutex) {
     _control->signal_task(val, _tag);
 }
 
-void TaskGroup::ready_to_run_general(bthread_t tid, bool nosignal) {
+void TaskGroup::ready_to_run_general(TaskMeta* meta, bool nosignal) {
     if (tls_task_group == this) {
-        return ready_to_run(tid, nosignal);
+        return ready_to_run(meta, nosignal);
     }
-    return ready_to_run_remote(tid, nosignal);
+    return ready_to_run_remote(meta, nosignal);
 }
 
 void TaskGroup::flush_nosignal_tasks_general() {
@@ -739,12 +792,16 @@ void TaskGroup::flush_nosignal_tasks_general() {
 
 void TaskGroup::ready_to_run_in_worker(void* args_in) {
     ReadyToRunArgs* args = static_cast<ReadyToRunArgs*>(args_in);
-    return tls_task_group->ready_to_run(args->tid, args->nosignal);
+    return tls_task_group->ready_to_run(args->meta, args->nosignal);
 }
 
 void TaskGroup::ready_to_run_in_worker_ignoresignal(void* args_in) {
     ReadyToRunArgs* args = static_cast<ReadyToRunArgs*>(args_in);
-    return tls_task_group->push_rq(args->tid);
+#ifdef BRPC_BTHREAD_TRACER
+    tls_task_group->_control->_task_tracer.set_status(
+        TASK_STATUS_READY, args->meta);
+#endif // BRPC_BTHREAD_TRACER
+    return tls_task_group->push_rq(args->meta->tid);
 }
 
 struct SleepArgs {
@@ -759,7 +816,7 @@ static void ready_to_run_from_timer_thread(void* arg) {
     const SleepArgs* e = static_cast<const SleepArgs*>(arg);
     auto g = e->group;
     auto tag = g->tag();
-    g->control()->choose_one_group(tag)->ready_to_run_remote(e->tid);
+    g->control()->choose_one_group(tag)->ready_to_run_remote(e->meta);
 }
 
 void TaskGroup::_add_sleep_event(void* void_args) {
@@ -768,6 +825,9 @@ void TaskGroup::_add_sleep_event(void* void_args) {
     // will be gone.
     SleepArgs e = *static_cast<SleepArgs*>(void_args);
     TaskGroup* g = e.group;
+#ifdef BRPC_BTHREAD_TRACER
+    g->_control->_task_tracer.set_status(TASK_STATUS_SUSPENDED, e.meta);
+#endif // BRPC_BTHREAD_TRACER
 
     TimerThread::TaskId sleep_id;
     sleep_id = get_global_timer_thread()->schedule(
@@ -777,7 +837,7 @@ void TaskGroup::_add_sleep_event(void* void_args) {
     if (!sleep_id) {
         e.meta->sleep_failed = true;
         // Fail to schedule timer, go back to previous thread.
-        g->ready_to_run(e.tid);
+        g->ready_to_run(e.meta);
         return;
     }
 
@@ -801,7 +861,7 @@ void TaskGroup::_add_sleep_event(void* void_args) {
         // schedule previous thread as well. If sleep_id does not exist,
         // previous thread is scheduled by timer thread before and we don't
         // have to do it again.
-        g->ready_to_run(e.tid);
+        g->ready_to_run(e.meta);
     }
 }
 
@@ -904,12 +964,12 @@ int TaskGroup::interrupt(bthread_t tid, TaskControl* c, bthread_tag_t tag) {
         if (get_global_timer_thread()->unschedule(sleep_id) == 0) {
             bthread::TaskGroup* g = bthread::tls_task_group;
             if (g) {
-                g->ready_to_run(tid);
+                g->ready_to_run(TaskGroup::address_meta(tid));
             } else {
                 if (!c) {
                     return EINVAL;
                 }
-                c->choose_one_group(tag)->ready_to_run_remote(tid);
+                c->choose_one_group(tag)->ready_to_run_remote(TaskGroup::address_meta(tid));
             }
         }
     }
@@ -918,7 +978,7 @@ int TaskGroup::interrupt(bthread_t tid, TaskControl* c, bthread_tag_t tag) {
 
 void TaskGroup::yield(TaskGroup** pg) {
     TaskGroup* g = *pg;
-    ReadyToRunArgs args = { g->current_tid(), false };
+    ReadyToRunArgs args = {  g->_cur_meta, false };
     g->set_remained(ready_to_run_in_worker, &args);
     sched(pg);
 }
@@ -940,6 +1000,9 @@ void print_task(std::ostream& os, bthread_t tid) {
     bool has_tls = false;
     int64_t cpuwide_start_ns = 0;
     TaskStatistics stat = {0, 0};
+    TaskStatus status = TASK_STATUS_UNKNOWN;
+    bool traced = false;
+    pid_t worker_tid = 0;
     {
         BAIDU_SCOPED_LOCK(m->version_lock);
         if (given_ver == *m->version_butex) {
@@ -953,6 +1016,9 @@ void print_task(std::ostream& os, bthread_t tid) {
             has_tls = m->local_storage.keytable;
             cpuwide_start_ns = m->cpuwide_start_ns;
             stat = m->stat;
+            status = m->status;
+            traced = m->traced;
+            worker_tid = m->worker_tid;
         }
     }
     if (!matched) {
@@ -969,7 +1035,15 @@ void print_task(std::ostream& os, bthread_t tid) {
            << "}\nhas_tls=" << has_tls
            << "\nuptime_ns=" << butil::cpuwide_time_ns() - cpuwide_start_ns
            << "\ncputime_ns=" << stat.cputime_ns
-           << "\nnswitch=" << stat.nswitch;
+           << "\nnswitch=" << stat.nswitch
+#ifdef BRPC_BTHREAD_TRACER
+           << "\nstatus=" << status
+           << "\ntraced=" << traced
+           << "\nworker_tid=" << worker_tid;
+#else
+           ;
+           (void)status;(void)traced;(void)worker_tid;
+#endif // BRPC_BTHREAD_TRACER
     }
 }
 
