@@ -61,7 +61,7 @@ int BAIDU_WEAK bthread_timed_connect(
 
 __END_DECLS
 
-#include "details/extended_endpoint.hpp"
+#include "butil/details/extended_endpoint.hpp"
 
 namespace butil {
 
@@ -419,18 +419,6 @@ short kqueue_to_poll_events(int kqueue_events) {
 
 int pthread_fd_wait(int fd, unsigned events,
                     const timespec* abstime) {
-    int diff_ms = -1;
-    if (abstime) {
-        timespec now;
-        clock_gettime(CLOCK_REALTIME, &now);
-        int64_t now_us = butil::timespec_to_microseconds(now);
-        int64_t abstime_us = butil::timespec_to_microseconds(*abstime);
-        if (abstime_us <= now_us) {
-            errno = ETIMEDOUT;
-            return -1;
-        }
-        diff_ms = (abstime_us - now_us + 999L) / 1000L;
-    }
 #if defined(OS_LINUX)
     const short poll_events = epoll_to_poll_events(events);
 #elif defined(OS_MACOSX)
@@ -441,13 +429,32 @@ int pthread_fd_wait(int fd, unsigned events,
         return -1;
     }
     pollfd ufds = { fd, poll_events, 0 };
-    const int rc = poll(&ufds, 1, diff_ms);
-    if (rc < 0) {
-        return -1;
+    int64_t abstime_us = -1;
+    if (NULL != abstime) {
+        abstime_us = butil::timespec_to_microseconds(*abstime);
     }
-    if (rc == 0) {
-        errno = ETIMEDOUT;
-        return -1;
+    while (true) {
+        int diff_ms = -1;
+        if (NULL != abstime) {
+            int64_t now_us = butil::gettimeofday_us();
+            if (abstime_us <= now_us) {
+                errno = ETIMEDOUT;
+                return -1;
+            }
+            diff_ms = (abstime_us - now_us + 999L) / 1000L;
+        }
+        int rc = poll(&ufds, 1, diff_ms);
+        if (rc > 0) {
+            break;
+        } else if (rc == 0) {
+            errno = ETIMEDOUT;
+            return -1;
+        } else {
+            if (errno == EINTR) {
+                continue;
+            }
+            return -1;
+        }
     }
     if (ufds.revents & POLLNVAL) {
         errno = EBADF;
@@ -458,10 +465,6 @@ int pthread_fd_wait(int fd, unsigned events,
 
 int pthread_timed_connect(int sockfd, const struct sockaddr* serv_addr,
                           socklen_t addrlen, const timespec* abstime) {
-    if (abstime == NULL) {
-        return ::connect(sockfd, serv_addr, addrlen);
-    }
-
     bool is_blocking = butil::is_blocking(sockfd);
     if (is_blocking) {
         butil::make_non_blocking(sockfd);
@@ -485,15 +488,7 @@ int pthread_timed_connect(int sockfd, const struct sockaddr* serv_addr,
         return -1;
     }
 
-    int err;
-    socklen_t errlen = sizeof(err);
-    if (getsockopt(sockfd, SOL_SOCKET, SO_ERROR, &err, &errlen) < 0) {
-        PLOG(FATAL) << "Fail to getsockopt";
-        return -1;
-    }
-    if (err != 0) {
-        CHECK(err != EINPROGRESS);
-        errno = err;
+    if (is_connected(sockfd) != 0) {
         return -1;
     }
     return 0;
@@ -513,22 +508,19 @@ int tcp_connect(const EndPoint& server, int* self_port, int connect_timeout_ms)
     if (sockfd < 0) {
         return -1;
     }
-    int rc = 0;
-    if (connect_timeout_ms <= 0) {
-        if (bthread_connect != NULL) {
-            rc = bthread_connect(sockfd, (struct sockaddr*)&serv_addr, serv_addr_size);
-        } else {
-            rc = ::connect(sockfd, (struct sockaddr*) &serv_addr, serv_addr_size);
-        }
+    timespec abstime{};
+    timespec* abstime_ptr = NULL;
+    if (connect_timeout_ms > 0) {
+        abstime = butil::milliseconds_from_now(connect_timeout_ms);
+        abstime_ptr = &abstime;
+    }
+    int rc;
+    if (bthread_timed_connect != NULL) {
+        rc = bthread_timed_connect(sockfd, (struct sockaddr*)&serv_addr,
+                                   serv_addr_size, abstime_ptr);
     } else {
-        timespec abstime = butil::milliseconds_from_now(connect_timeout_ms);
-        if (bthread_timed_connect != NULL) {
-            rc = bthread_timed_connect(sockfd, (struct sockaddr*)&serv_addr,
-                                       serv_addr_size, &abstime);
-        } else {
-            rc = pthread_timed_connect(sockfd, (struct sockaddr*) &serv_addr,
-                                       serv_addr_size, &abstime);
-        }
+        rc = pthread_timed_connect(sockfd, (struct sockaddr*) &serv_addr,
+                                   serv_addr_size, abstime_ptr);
     }
     if (rc < 0) {
         return -1;
